from src.license_scanner.get_all_licenses import get_licenses, parse_license
from src.license_scanner.parse_license.licenses_synonyms import (
    mit_license,
    bsd_license,
    apache_license_v2,
    apache_license,
    mit_cmu,
    historical_permission_notice,
)
import pytest
import sys
import re

PYTHON_MINOR_VERSION = int(re.findall(r"3.(\d+).\d+", sys.version)[0])


@pytest.mark.parametrize("model_name", ["mergedeep", "jaraco.text"])
def test_get_licenses_mit(model_name):
    licenses = get_licenses(model_name)
    licenses = [parse_license(i) for i in licenses]
    assert mit_license in licenses


@pytest.mark.parametrize("model_name", ["numpy", "scikit_image"])
def test_get_licenses_general_bsd(model_name):
    licenses = get_licenses(model_name)
    licenses = [parse_license(i) for i in licenses]
    assert bsd_license in licenses


@pytest.mark.parametrize("model_name", ["absl_py"])
def test_get_licenses_apache(model_name):
    licenses = get_licenses(model_name)
    licenses = [parse_license(i) for i in licenses]
    assert apache_license_v2 in licenses


def test_get_dual_license():
    licenses = get_licenses("uv")
    licenses = [parse_license(i) for i in licenses]
    assert mit_license in licenses
    assert apache_license in licenses


def test_get_licenses_pillow():
    licenses = get_licenses("pillow")
    licenses = [parse_license(i) for i in licenses]

    # Pillow change there license in version 11
    # Python 3.13 and up will have the new version of Pillow
    if PYTHON_MINOR_VERSION <= 12:
        assert historical_permission_notice in licenses
    else:
        assert mit_cmu in licenses



import warnings
from src.license_scanner.parse_license import parse_license
from src.license_scanner.parse_license.licenses_synonyms import (
    unknown_license,
    mit_license,
    gnu_lesser_general_public_license_v2_0_only,
    apache_license_v2,
)


def test_parse_license_empty_string():
    assert parse_license("") is None


def test_parse_license_none():
    assert parse_license(None) is None


def test_parse_license_with_newline():
    license_text = "This is a full license\nwith multiple lines\nand more content"

    with warnings.catch_warnings(record=True) as w:
        warnings.simplefilter("always")
        result = parse_license(license_text)

        assert result is None
        assert len(w) == 1
        assert "It seems like a full license" in str(w[0].message)


def test_parse_license_strips_whitespace():
    result = parse_license("  MIT  ")
    assert result == mit_license


def test_parse_license_mit():
    result = parse_license("MIT")
    assert result == mit_license


def test_parse_gnu_lesser_gpl_v2():
    result = parse_license("GNU Lesser General Public License v2.0 only")
    assert result == gnu_lesser_general_public_license_v2_0_only


def test_parse_license_handles_copyright():
    result = parse_license("MIT copyright (c) 2023 John Doe")
    assert result == mit_license


def test_parse_license_not_found_in_synonyms():
    with warnings.catch_warnings(record=True) as w:
        warnings.simplefilter("always")
        result = parse_license("a License not in the list")

        assert result == "a license not in the list"
        assert len(w) == 1
        assert 'The license "a license not in the list" was not found' in str(
            w[0].message
        )


def test_parse_license_not_found_in_synonyms_normalized():
    with warnings.catch_warnings(record=True) as w:
        warnings.simplefilter("always")
        license1 = "a License not in the list"
        license2 = "   A LiceNse not in the lisT   "
        result1 = parse_license(license1)
        result2 = parse_license(license2)

        assert result1 == "a license not in the list"
        assert result2 == result1
        assert len(w) == 2
        assert f'The license "{license1.lower().strip()}" was not found' in str(
            w[0].message
        )
        assert f'The license "{license2.lower().strip()}" was not found' in str(
            w[1].message
        )


def test_parse_license_case_insensitive():
    result = parse_license("APACHE 2.0")
    assert result == apache_license_v2


def test_parse_license_empty_after_processing():
    result = parse_license("Copyright (c) 2023")
    assert result == unknown_license



import re
from typing import List
import warnings
from enum import Enum
from ..parse_license import parse_license
from ..parse_license.licenses_synonyms import LICENSES_SYNONYMS


class SpdxOperator(Enum):
    AND = "AND"
    OR = "OR"
    ACCEPTED = "ACCEPTED"
    REJECTED = "REJECTED"


def _split_expression(license_expression: str) -> List[str]:
    """Split the license expression into parts based on the operators found (AND/OR).
        For example
        'Q OR (A AND (B OR C)) AND D'
        becomes
        ['Q', 'OR', 'A AND (B OR C)', 'AND', 'D']

    :param license_expression: The SPDX license expression to split.
    :type license_expression: str
    :return: List containing the parts of the expression split by the operators.
    :rtype: List[str]
    """

    # Dealing with brackets
    if "(" in license_expression:
        # Number opening and closing brackets should match
        if license_expression.count("(") != license_expression.count(")"):
            warnings.warn("Unmatched parentheses in license expression.", UserWarning)
            return [license_expression]

        # Find the first opening bracket
        search_opening_bracket = re.search(r"\(", license_expression)
        start_index = search_opening_bracket.start()
        end_index = search_opening_bracket.end()
        number_of_brackets = 1

        # Find the matching closing bracket
        # Handle nested brackets as well;
        # e.g., Q OR (A AND (B OR C)) OR Z
        # Will return the indexes "(A AND (B OR C))"
        while number_of_brackets > 0:
            next_char = license_expression[end_index]
            if next_char == "(":
                number_of_brackets += 1
            elif next_char == ")":
                number_of_brackets -= 1
            end_index += 1

        # Split the string in 3 parts, before, between and after the brackets
        first_part = license_expression[:start_index]
        part_between_brackets = license_expression[start_index + 1 : end_index - 1]
        last_part = license_expression[end_index:]

        # Recursively split the parts
        return_values = []
        if first_part.strip():
            # If there is something before the brackets
            # It will still need to be split
            # because even with brackets there can be operators (AND, OR) before
            return_values += _split_expression(first_part)
        return_values.append(part_between_brackets.strip())
        if last_part.strip():
            # Something after the brackets can have brackets too
            # So recursively split it
            return_values += _split_expression(last_part)
        return return_values

    license_expression_norm = license_expression.lower()
    pos_first_and = license_expression_norm.find(" and ")
    pos_first_or = license_expression_norm.find(" or ")

    if pos_first_and != -1 and (pos_first_and < pos_first_or or pos_first_or == -1):
        # First operator is AND
        first_part = license_expression[:pos_first_and]
        second_part = license_expression[pos_first_and + 5 :]
        return_values = [first_part, "AND"] + _split_expression(second_part)
    elif pos_first_or != -1 and (pos_first_or < pos_first_and or pos_first_and == -1):
        # First operator is OR
        first_part = license_expression[:pos_first_or]
        second_part = license_expression[pos_first_or + 4 :]
        return_values = [first_part, "OR"] + _split_expression(second_part)
    else:
        return_values = [license_expression]

    return_values = [part for part in return_values if part]
    return return_values


def spdx_logic(license_expression: str, allowed_licenses: List[str]) -> bool:
    """Evaluate the SPDX license expression against the allowed licenses.
    These are expressions like "MIT OR Apache-2.0" or "GPL-3.0 AND (BSD-3-Clause OR MIT)".

    This follows: https://spdx.github.io/spdx-spec/v2.3/SPDX-license-expressions/

    :param license_expression: The SPDX license expression to evaluate.
    :type license_expression: str
    :param allowed_licenses: The list of allowed licenses.
    :type allowed_licenses: List[str]
    :return: True if the license expression is valid, False otherwise.
    :rtype: bool
    """

    allowed_licenses = [parse_license(lic) for lic in allowed_licenses]

    license_expression = license_expression.lower().strip()

    if license_expression in LICENSES_SYNONYMS.keys():
        # If the license (including operators)is a known synonym we do NOT process it further
        # e.g. "Historical Permission Notice and Disclaimer (HPND)"
        parsed_license = parse_license(license_expression)
        return parsed_license in allowed_licenses

    split_expression = _split_expression(license_expression)

    if len(split_expression) == 1:
        # Single license
        parsed_license = parse_license(split_expression[0])
        return parsed_license in allowed_licenses

    # Process all parts that are not operators
    processed_parts: List[SpdxOperator] = []
    for part in split_expression:
        if part.lower() == "and":
            processed_parts.append(SpdxOperator.AND)
        elif part.lower() == "or":
            processed_parts.append(SpdxOperator.OR)
        else:
            processed_part = spdx_logic(part, allowed_licenses)
            if processed_part:
                processed_parts.append(SpdxOperator.ACCEPTED)
            else:
                processed_parts.append(SpdxOperator.REJECTED)

    # Now evaluate the processed parts
    # First handle all ANDs
    while SpdxOperator.AND in processed_parts:
        index_and = processed_parts.index(SpdxOperator.AND)
        if index_and == 0 or index_and == len(processed_parts) - 1:
            warnings.warn(
                f"Malformed SPDX expression: AND operator at the beginning or end: {license_expression}",
                UserWarning,
            )
            return False
        left_part = processed_parts[index_and - 1]
        right_part = processed_parts[index_and + 1]

        if left_part == SpdxOperator.ACCEPTED and right_part == SpdxOperator.ACCEPTED:
            result = SpdxOperator.ACCEPTED
        else:
            result = SpdxOperator.REJECTED

        # Replace the three parts with the result
        processed_parts = (
            processed_parts[: index_and - 1]
            + [result]
            + processed_parts[index_and + 2 :]
        )

    # Then handle all ORs
    while SpdxOperator.OR in processed_parts:
        index_or = processed_parts.index(SpdxOperator.OR)
        if index_or == 0 or index_or == len(processed_parts) - 1:
            warnings.warn(
                f"Malformed SPDX expression: OR operator at the beginning or end: {license_expression}",
                UserWarning,
            )
            return False
        left_part = processed_parts[index_or - 1]
        right_part = processed_parts[index_or + 1]

        if left_part == SpdxOperator.ACCEPTED or right_part == SpdxOperator.ACCEPTED:
            result = SpdxOperator.ACCEPTED
        else:
            result = SpdxOperator.REJECTED

        # Replace the three parts with the result
        processed_parts = (
            processed_parts[: index_or - 1] + [result] + processed_parts[index_or + 2 :]
        )

    # At the end there should be only one part left
    if len(processed_parts) == 1:
        return processed_parts[0] == SpdxOperator.ACCEPTED
    else:
        warnings.warn(
            f"Malformed SPDX expression after processing: {license_expression}",
            UserWarning,
        )
        return False



import pandas as pd
from typing import List
from pandas import read_html
import requests
from license_scanner.parse_license import parse_license
from license_scanner.parse_license.licenses_synonyms import LICENSES_SYNONYMS

license_url = "https://spdx.org/licenses/"


def get_spdx_licenses() -> List[List[str]]:
    """
    Get all SPDX licenses from the SPDX website.
    """
    # Get the HTML content of the page
    response = requests.get(license_url)
    html_content = response.content

    # Parse the HTML content using pandas
    tables = read_html(html_content, flavor="bs4")

    _licenses = []
    # First table contains the license data
    license_table = tables[0]

    # Second table contains Deprecated License Identifiers
    license_table_depracted = tables[1]

    full_table = pd.concat(
        [
            license_table[["Full name", "Identifier"]],
            license_table_depracted[["Full name", "Identifier"]],
        ]
    )
    # Extract the license IDs and names
    for row in full_table.iterrows():
        license_name = row[1]["Full name"]
        license_id = row[1]["Identifier"]
        license_parsed = parse_license(license_name)
        license_id_parsed = parse_license(license_id)
        if (
            LICENSES_SYNONYMS.get(license_parsed.lower()) is None
            or LICENSES_SYNONYMS.get(license_id_parsed.lower()) is None
        ):
            _licenses.append([license_name, license_id])
    return _licenses


def create_variable_name(license_id: str) -> str:
    """
    Create a variable name from the license ID.
    """
    variable_name = license_id.lower().replace("-", "_").replace(".", "_")
    if variable_name[0].isdigit():
        variable_name = "_" + variable_name
    return variable_name


licenses = get_spdx_licenses()

if len(licenses) == 0:
    print("All licenses found in license_scanner")
else:
    print(f"{len(licenses)} licenses not found in license_scanner")
    # Print for readme
    print("\nREADME\n")
    for _license in licenses:
        print(f"- {_license[0]}")
    # Print new variables
    print("\nVariables\n")

    for _license in licenses:
        variable_name = create_variable_name(_license[1])
        print(f'{variable_name} = "{_license[0]}"')
    print("\nSynonyms\n")
    # Print first lines of LICENSES_SYNONYMS
    for _license in licenses:
        variable_name = create_variable_name(_license[1])
        print(f"{variable_name}.lower() : {variable_name},")
    print("\nIDs\n")
    # add ids
    for _license in licenses:
        variable_name = create_variable_name(_license[1])
        print(f'"{_license[1].lower()}" : {variable_name},')
    exit(1)



from src.license_scanner.spdx_logic.main import _split_expression


def test_split_expression_happy_flow1():

    assert _split_expression("MIT AND Apache-2.0") == [
        "MIT",
        "AND",
        "Apache-2.0",
    ]


def test_split_expression_happy_flow2():

    assert _split_expression("MIT OR Apache-2.0") == [
        "MIT",
        "OR",
        "Apache-2.0",
    ]


def test_split_expression_happy_flow3():

    assert _split_expression("MIT") == ["MIT"]


def test_split_expression_happy_flow4():

    assert _split_expression("MIT AND Apache-2.0 OR BSD-3-Clause") == [
        "MIT",
        "AND",
        "Apache-2.0",
        "OR",
        "BSD-3-Clause",
    ]


def test_split_expression_happy_flow5():

    assert _split_expression("MIT OR Apache-2.0 AND BSD-3-Clause") == [
        "MIT",
        "OR",
        "Apache-2.0",
        "AND",
        "BSD-3-Clause",
    ]


def test_split_partly_expression1():

    assert _split_expression(" AND Apache-2.0") == [
        "AND",
        "Apache-2.0",
    ]


def test_split_partly_expression2():
    assert _split_expression(" OR Apache-2.0") == [
        "OR",
        "Apache-2.0",
    ]


def test_split_partly_expression3():
    assert _split_expression("") == []


def test_split_partly_expression4():
    assert _split_expression("MIT AND Apache-2.0 OR ") == [
        "MIT",
        "AND",
        "Apache-2.0",
        "OR",
    ]


def test_split_partly_expression5():
    assert _split_expression("MIT OR Apache-2.0 AND ") == [
        "MIT",
        "OR",
        "Apache-2.0",
        "AND",
    ]


def test_split_expression_pure_brackets():
    assert _split_expression("(MIT AND Apache-2.0)") == ["MIT AND Apache-2.0"]


def test_split_expression_brackets_and_not():

    assert _split_expression("(MIT OR Apache-2.0) AND BSD-3-Clause") == [
        "MIT OR Apache-2.0",
        "AND",
        "BSD-3-Clause",
    ]


def test_split_expression_brackets_nested_brackets():

    assert _split_expression(
        "BSD-3-Clause AND ((MIT OR BSD-3-Clause) OR Apache-2.0)"
    ) == ["BSD-3-Clause", "AND", "(MIT OR BSD-3-Clause) OR Apache-2.0"]


def test_split_expression_multiple_brackets():

    assert _split_expression("(MIT OR Apache-2.0) OR (BSD-3-Clause AND MIT)") == [
        "MIT OR Apache-2.0",
        "OR",
        "BSD-3-Clause AND MIT",
    ]


def test_split_expression_multiple_and_nested_brackets():

    assert _split_expression(
        "(MIT OR Apache-2.0) OR ((MIT OR BSD-3-Clause) OR Apache-2.0)"
    ) == ["MIT OR Apache-2.0", "OR", "(MIT OR BSD-3-Clause) OR Apache-2.0"]



from unittest.mock import patch

import pytest
from src.license_scanner.cli import main, Mode

PYTEST_PACKAGES = [
    "colorama",
    "exceptiongroup",
    "iniconfig",
    "packaging",
    "pluggy",
    "pygments",
    "pytest",
    "tomli",
    "typing_extensions",
    "autocommand",
]


@pytest.mark.parametrize(
    "allowed_licenses, allowed_packages",
    [
        (
            [
                "BSD license",
                "BSD 2-clause license",
                "BSD 3-clause license",
                "MiT",
                "Apache-2.0",
                "Apache license",
                "Historical Permission Notice and Disclaimer (HPND)",
                "CMU License (MIT-CMU)",
            ],
            [*PYTEST_PACKAGES],
        ),
        (
            [
                "BSD license",
                "BSD 2-clause license",
                "BSD 3-clause license",
                "MiT",
                "Apache license",
                "Apache-2.0",
            ],
            ["pillow", *PYTEST_PACKAGES],
        ),
        (
            [
                "BSD license",
                "BSD 2-clause license",
                "BSD 3-clause license",
                "Apache license",
                "MiT",
                "Historical Permission Notice and Disclaimer (HPND)",
                "CMU License (MIT-CMU)",
            ],
            ["absl_py", *PYTEST_PACKAGES],
        ),
        (
            [
                "BSD license",
                "BSD 2-clause license",
                "BSD 3-clause license",
                "Apache license",
                "MiT",
                "Historical Permission Notice and Disclaimer (HPND)",
                "CMU License (MIT-CMU)",
            ],
            ["absl-py", *PYTEST_PACKAGES],
        ),
    ],
    ids=[
        "pure allowed licenses",
        "no HPND or MIT-CMU",
        "no Apache 2.0",
        "no Apache 2.0 used - in package name",
    ],
)
@patch("src.license_scanner.cli.__get_arguments")
@patch("src.license_scanner.cli.tomllib.load")
def test_whitelist_integration_passing(
    toml_load_mock,
    argparse_mock,
    allowed_licenses,
    allowed_packages,
):
    toml_load_mock.return_value = {
        "tool": {
            "license_scanner": {
                "allowed_licenses": allowed_licenses,
                "allowed_packages": allowed_packages,
            }
        }
    }
    argparse_mock.return_value = Mode.whitelist
    main()


@pytest.mark.parametrize(
    "allowed_licenses, allowed_packages",
    [
        (
            [
                "BSD license",
                "BSD 2-clause license",
                "BSD 3-clause license",
                "Apache-2.0",
                "Apache license",
                "Historical Permission Notice and Disclaimer (HPND)",
                "CMU License (MIT-CMU)",
            ],
            [*PYTEST_PACKAGES],
        ),
        (
            [
                "BSD license",
                "BSD 2-clause license",
                "BSD 3-clause license",
                "MiT",
                "Apache license",
                "Apache-2.0",
            ],
            [*PYTEST_PACKAGES],
        ),
        (
            [
                "BSD license",
                "BSD 2-clause license",
                "BSD 3-clause license",
                "Apache license",
                "MiT",
                "Historical Permission Notice and Disclaimer (HPND)",
                "CMU License (MIT-CMU)",
            ],
            [*PYTEST_PACKAGES],
        ),
    ],
    ids=[
        "no MIT",
        "no HPND or MIT-CMU",
        "no Apache 2.0",
    ],
)
@patch("src.license_scanner.cli.__get_arguments")
@patch("src.license_scanner.cli.tomllib.load")
def test_whitelist_integration_failing(
    toml_load_mock,
    argparse_mock,
    allowed_licenses,
    allowed_packages,
):
    toml_load_mock.return_value = {
        "tool": {
            "license_scanner": {
                "allowed_licenses": allowed_licenses,
                "allowed_packages": allowed_packages,
            }
        }
    }
    argparse_mock.return_value = Mode.whitelist
    with pytest.raises(ValueError):
        main()



import argparse
import sys
from enum import Enum
from pathlib import Path
from typing import TYPE_CHECKING, Any, Dict

from .get_all_licenses import get_all_licenses
from .parse_license import parse_license
from .parse_license.licenses_synonyms import unknown_license
from .spdx_logic.main import spdx_logic

if sys.version_info >= (3, 11):
    try:
        import tomllib
    except ImportError:
        # Help users on older alphas
        if not TYPE_CHECKING:
            import tomli as tomllib
else:
    import tomli as tomllib


class Mode(Enum):
    whitelist = "whitelist"
    print = "print"
    blacklist = "blacklist"


def parse_pyproject_toml() -> Dict[str, Any]:
    """Parse a pyproject toml file, pulling out relevant parts for Black.

    If parsing fails, will raise a tomllib.TOMLDecodeError.
    """
    path_pyproject_toml = Path.cwd() / "pyproject.toml"
    assert path_pyproject_toml.exists(), f"{path_pyproject_toml} file not found"
    with open(path_pyproject_toml, "rb") as f:
        pyproject_toml = tomllib.load(f)
    config: Dict[str, Any] = pyproject_toml.get("tool", {}).get("license_scanner", {})
    config = {k.replace("--", "").replace("-", "_"): v for k, v in config.items()}

    return config


def __get_arguments() -> Mode:
    parser = argparse.ArgumentParser(
        description="Just an example",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument(
        "-m",
        "--mode",
        type=Mode,
        choices=list(Mode),
        help=f"The mode determines what to do with the results. Options are: {list(Mode)}",
        default=Mode.print,
    )
    args = parser.parse_args()
    config = vars(args)
    mode = config["mode"]
    return mode


def main():
    """
    Get all the installed packages and licenses.
    If will use the system argument -m or --mode to determine what to do with it.

    Mode: print (default)
        Prints the packages sorted by license
    Mode: Whitelist
        Gives an error if one of packages and it licenses are not whitelisted
        Whitelist are loaded from pyproject.toml -- allowed-licenses && allowed-packages
    """

    mode = __get_arguments()

    all_licenses = get_all_licenses()

    all_used_licenses = list(all_licenses.keys())
    all_used_licenses.sort()

    if mode == Mode.print:
        for key in all_used_licenses:
            print(f"\n ======\n {key} \n ======")
            for _license in all_licenses[key]:
                print(f" - {_license}")

    elif mode == Mode.whitelist:
        pyproject_config = parse_pyproject_toml()

        raw_allowed_licenses = pyproject_config.get("allowed_licenses", [])
        allowed_licenses = [parse_license(i) for i in raw_allowed_licenses]
        if None in all_licenses:
            allowed_licenses.remove(None)
        raw_allowed_packages = pyproject_config.get("allowed_packages", [])
        allowed_packages = [i.lower().replace("-", "_") for i in raw_allowed_packages]

        # Check if all license or package are in a allowed list
        problem_packages = []
        for used_license in all_used_licenses:
            if not spdx_logic(used_license, allowed_licenses):
                for package in all_licenses[used_license]:
                    # package "license_scanner" and "license-scanner" are the same
                    if package.replace("-", "_") not in allowed_packages:
                        problem_packages.append((package, used_license))

        # If any package is not allowed raise error
        if problem_packages:
            for package, license in problem_packages:
                if license == unknown_license:
                    print(
                        f"{package} license could not be found. If this is a error please report at https://github.com/wagenrace/license_scanner/issues if you want to ignore this package add it to the whitelist in pyproject.toml"
                    )
                else:
                    print(f'{package} with "{license}" is NOT whitelisted')
            raise ValueError(
                "Some of the packages found do not use white listed licenses"
            )
        else:
            print("=+=+=+=+= All packages are good to use =+=+=+=+=")


if __name__ == "__main__":
    main()



from src.license_scanner.get_all_packages import get_all_package_names
import pytest


@pytest.mark.parametrize(
    "model_name",
    ["numpy", "mergedeep", "absl_py", "scikit_image", "jaraco.text", "pillow", "uv"],
)
def test_get_all_package_names(model_name):
    packages = get_all_package_names()
    assert model_name in packages



# %%
from typing import Dict, List

from .parse_license import parse_license
from .parse_license.licenses_synonyms import unknown_license
from .get_all_packages import get_all_package_names

import importlib.metadata as im


def get_licenses(name: str) -> List[str]:
    """Get the licenses for a given package.
    This will return the raw license strings, which can be parsed with parse_license.
    Only "UNKNOWN" in the licenses field will be skipped.

    :param name: The name of the package
    :type name: str
    :return: A list of licenses for the package
    :rtype: List[str]
    """
    all_licenses = []
    try:
        metas = im.metadata(name)
    except im.PackageNotFoundError:
        return [unknown_license]
    for key, value in metas.items():
        if key == "License-Expression":
            license = value
            if license:
                all_licenses.append(license)
        if key == "License":
            license = value
            if license == "UNKNOWN":
                # Sometimes this is unknown because it is not filled in
                continue
            if license:
                all_licenses.append(license)
        if key == "Classifier":
            if not value.startswith("License ::"):
                continue
            license = value.split("::")[-1]
            if license.strip() == "OSI Approved":
                continue
            if license:
                all_licenses.append(license)

    all_licenses = all_licenses
    return all_licenses


def get_all_licenses() -> Dict[str, str]:
    """
    Get all packages installed in the environment and their licenses.
    This will return a dictionary with the license as key and a list of packages names as value.

    :return: A dictionary with the license as key and a list of packages names as value
    :rtype: Dict[str, str]
    """
    all_licenses = {}
    all_package_names = get_all_package_names()

    for package_name in all_package_names:
        licenses_raw = get_licenses(package_name)

        # Parse and remove duplicates
        licenses = list(set([parse_license(i) for i in licenses_raw]))
        licenses = [i for i in licenses if i]

        # If no license is known let it know
        if len(licenses) == 0:
            licenses = [unknown_license]

        # Reformat output
        for _license in licenses:
            all_licenses[_license] = all_licenses.get(_license, []) + [package_name]

    return all_licenses



import pytest
from src.license_scanner.spdx_logic.main import spdx_logic


def test_spdx_single_license_no_MIT():
    allowed_licenses = [
        "Apache-2.0",
        "BSD-3-Clause",
        "completely unknown license",
    ]

    assert spdx_logic("MIT", allowed_licenses) is False


def test_spdx_single_license_MIT():
    allowed_licenses = [
        "MIT",
        "Apache-2.0",
        "BSD-3-Clause",
        "completely unknown license",
    ]
    assert spdx_logic("MIT", allowed_licenses) is True


def test_spdx_single_license_Apache_2_0():
    allowed_licenses = [
        "MIT",
        "Apache-2.0",
        "BSD-3-Clause",
        "completely unknown license",
    ]
    assert spdx_logic("Apache-2.0", allowed_licenses) is True


def test_spdx_single_license_BSD_3_Clause():
    allowed_licenses = [
        "MIT",
        "Apache-2.0",
        "BSD-3-Clause",
        "completely unknown license",
    ]
    assert spdx_logic("BSD-3-Clause", allowed_licenses) is True


def test_spdx_single_license_GPL_3_0():
    allowed_licenses = [
        "MIT",
        "Apache-2.0",
        "BSD-3-Clause",
        "completely unknown license",
    ]
    assert spdx_logic("GPL-3.0", allowed_licenses) is False

    # License NOT named in SPDX, but present in allowed licenses


def test_spdx_single_license_MIT_license():
    allowed_licenses = [
        "MIT",
        "Apache-2.0",
        "BSD-3-Clause",
        "completely unknown license",
    ]
    assert spdx_logic("MIT license", allowed_licenses) is True


def test_spdx_single_license_BSD_3_Clause_license():
    allowed_licenses = [
        "MIT",
        "Apache-2.0",
        "BSD-3-Clause",
        "completely unknown license",
    ]
    assert spdx_logic("BSD 3-clause license", allowed_licenses) is True


def test_spdx_single_license_GPL_version_3():
    allowed_licenses = [
        "MIT",
        "Apache-2.0",
        "BSD-3-Clause",
        "completely unknown license",
    ]
    assert spdx_logic("GPL version 3", allowed_licenses) is False


def test_spdx_single_license_mit_0():
    allowed_licenses = [
        "MIT",
        "Apache-2.0",
        "BSD-3-Clause",
        "completely unknown license",
    ]
    assert spdx_logic("mit 0", allowed_licenses) is False

    # License NOT named in SPDX and NOT present in allowed licenses


def test_spdx_single_license_completely_unknown_license():
    allowed_licenses = [
        "MIT",
        "Apache-2.0",
        "BSD-3-Clause",
        "completely unknown license",
    ]
    assert spdx_logic("completely unknown license", allowed_licenses) is True


def test_spdx_single_license_another_unknown_license():
    allowed_licenses = [
        "MIT",
        "Apache-2.0",
        "BSD-3-Clause",
        "completely unknown license",
    ]
    assert spdx_logic("another unknown license", allowed_licenses) is False


def test_spdx_simple_expressions_or_both_allowed():
    allowed_licenses = ["MIT", "Apache-2.0", "BSD-3-Clause"]

    assert spdx_logic("MIT OR Apache-2.0", allowed_licenses) is True


def test_spdx_simple_expressions_or_one_allowed():
    allowed_licenses = ["MIT", "BSD-3-Clause"]
    assert spdx_logic("MIT OR GPL-3.0", allowed_licenses) is True


def test_spdx_simple_expressions_or_other_one_allowed():
    allowed_licenses = ["MIT", "Apache-2.0", "BSD-3-Clause"]
    assert spdx_logic("GPL-3.0 OR Apache-2.0", allowed_licenses) is True


def test_spdx_simple_expressions_or_none_allowed():
    allowed_licenses = ["MIT", "Apache-2.0", "BSD-3-Clause"]
    assert spdx_logic("BSD-4-Clause OR GPL-3.0", allowed_licenses) is False


def test_spdx_simple_expressions_and_both_allowed():
    allowed_licenses = ["MIT", "Apache-2.0", "BSD-3-Clause"]
    assert spdx_logic("MIT AND BSD-3-Clause", allowed_licenses) is True


def test_spdx_simple_expressions_and_one_allowed():
    allowed_licenses = ["MIT", "Apache-2.0", "BSD-3-Clause"]
    assert spdx_logic("MIT AND GPL-3.0", allowed_licenses) is False


def test_spdx_simple_expressions_and_other_one_allowed():
    allowed_licenses = ["MIT", "Apache-2.0", "BSD-3-Clause"]
    assert spdx_logic("GPL-3.0 AND BSD-3-Clause", allowed_licenses) is False


def test_spdx_multiple_operators_expressions_only_or_all_allowed():

    allowed_licenses = ["MIT", "Apache-2.0", "BSD-3-Clause"]

    assert spdx_logic("MIT OR Apache-2.0 OR BSD-3-Clause", allowed_licenses) is True


def test_spdx_multiple_operators_expressions_all_or_some_allowed():
    allowed_licenses = ["MIT", "Apache-2.0", "BSD-3-Clause"]
    assert spdx_logic("MIT OR GPL-3.0 OR BSD-3-Clause", allowed_licenses) is True


def test_spdx_multiple_operators_expressions_all_or_none_allowed():
    allowed_licenses = ["MIT", "Apache-2.0", "BSD-3-Clause"]
    assert spdx_logic("GPL-2.0 OR GPL-3.0 OR LGPL-2.1", allowed_licenses) is False


def test_spdx_multiple_operators_expressions_all_and_all_allowed():
    allowed_licenses = ["MIT", "Apache-2.0", "BSD-3-Clause"]
    assert spdx_logic("MIT AND Apache-2.0 AND BSD-3-Clause", allowed_licenses) is True


def test_spdx_multiple_operators_expressions_all_and_some_allowed():
    allowed_licenses = ["MIT", "Apache-2.0", "BSD-3-Clause"]
    assert spdx_logic("MIT AND Apache-2.0 AND GPL-3.0", allowed_licenses) is False


@pytest.mark.parametrize(
    "license_expression",
    [
        "MIT AND GPL-3.0 OR BSD-3-Clause",
        "GPL-2.0 AND GPL-3.0 OR BSD-3-Clause",
        "BSD-3-Clause OR GPL-2.0 AND GPL-3.0",
    ],
)
def test_spdx_multiple_operators_expressions_and_or_combined_positive(
    license_expression,
):
    # D.4.5 Order of precedence and parentheses
    # AND should be resolved before OR

    allowed_licenses = ["MIT", "Apache-2.0", "BSD-3-Clause"]
    assert spdx_logic(license_expression, allowed_licenses) is True


@pytest.mark.parametrize(
    "license_expression",
    ["GPL-2.0 OR GPL-3.0 AND BSD-3-Clause", "GPL-3.0 AND BSD-3-Clause OR GPL-2.0"],
)
def test_spdx_multiple_operators_expressions_and_or_combined_negative(
    license_expression,
):
    # D.4.5 Order of precedence and parentheses
    # AND should be resolved before OR

    allowed_licenses = ["MIT", "Apache-2.0", "BSD-3-Clause"]
    assert spdx_logic(license_expression, allowed_licenses) is False


@pytest.mark.parametrize(
    "license_expression",
    [
        "(MIT OR Apache-2.0) AND BSD-3-Clause",
        "BSD-3-Clause AND (MIT OR Apache-2.0)",
        "(MIT OR GPL-3.0) AND BSD-3-Clause",
        "BSD-3-Clause AND (MIT OR GPL-3.0)",
    ],
)
def test_spdx_single_level_brackets_expressions_pass(license_expression):
    allowed_licenses = ["MIT", "Apache-2.0", "BSD-3-Clause"]

    assert spdx_logic(license_expression, allowed_licenses) is True


@pytest.mark.parametrize(
    "license_expression",
    [
        "(MIT OR BSD-3-Clause) AND GPL-3.0",
        "GPL-3.0 AND (MIT OR BSD-3-Clause)",
        "GPL-3.0 OR (GPL-2.0 AND BSD-3-Clause)",
        "MIT AND (GPL-2.0 OR GPL-3.0)",
    ],
)
def test_spdx_single_level_brackets_expressions_fail(license_expression):
    allowed_licenses = ["MIT", "Apache-2.0", "BSD-3-Clause"]

    assert spdx_logic(license_expression, allowed_licenses) is False


@pytest.mark.parametrize(
    "license_name",
    [
        'BSD 4-Clause "Original" or "Old" License',
        'BSD 3-Clause "New" or "Revised" License',
        "GNU Affero General Public License v1.0 or later",
        "Historical Permission Notice and Disclaimer (HPND)",
        "FSF Unlimited License (With License Retention and Warranty Disclaimer)",
    ],
)
def test_spdx_licenses_with_and_or_in_name(license_name):
    allowed_licenses = [license_name]

    assert spdx_logic(license_name, allowed_licenses) is True
